/*  Ec      Copyright (C)   2001 Marc Dechico

    ec_data.h : a common header file where global datas and functions
		are defined. 
		(working to change most of  them to static )

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    To contact the author:	Marc Dechico
			        Kerfring
			        29390 SCAER France
				tel: 06 80 83 74 99
				Mail: Marc.DECHICO@wanadoo.fr
    
*/

/* AIX requires this to be the first thing in the file.  */
#if defined (_AIX) && !defined (__GNUC__)
 #pragma alloca
#endif
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <getopt.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <errno.h>


#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#if ENABLE_NLS
#include <libintl.h>
#define _(String) gettext (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#else
#define _(String) (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#endif
#ifndef errno
extern int errno;
#endif

#ifdef	STDC_HEADERS
#include <stdlib.h>
#else	/* Not STDC_HEADERS */
extern void exit ();
extern char *malloc ();
#endif	/* STDC_HEADERS */

#ifdef	HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif

#ifdef	__GNUC__
#undef	alloca
#define	alloca(n)	__builtin_alloca (n)
#else	/* Not GCC.  */
#ifdef	HAVE_ALLOCA_H
#include <alloca.h>
#else	/* Not HAVE_ALLOCA_H.  */
#ifndef	_AIX
extern char *alloca ();
#endif	/* Not _AIX.  */
#endif	/* HAVE_ALLOCA_H.  */
#endif	/* GCC.  */

#ifdef HAVE_SYS_PARAM_H
/* To possibly get the definition of DEV_BSIZE. */
#include <sys/param.h>
#endif
#include "system.h"

#include	<ctype.h>
#include 	<signal.h>
#include 	<time.h>  

#include 	"ec_defin.i"


#define ST_NOT_MVT_INI   struct	 st_not_mvt_ini  



/* fonction mouvement */
typedef void (*tpf_xxx)(void);


/*****************************************************************************/
/* Structure des donnes                                                     */
/*****************************************************************************/
/*   tableau des pices un par camp  */
/* opti: voir si un seul tableau possible */
struct st_pie
{
  int i_Id;				/* Id de la piece */	
  int i_pie_sui;		        /* pice suivante */
  int i_pie_pre;                        /* pice precedente  */
  int i_pos_n;                          /* position chiquier  */
  int i_for; 			        /* force  */
  int i_for_inc;                        /* force increment pour eviter sacrifice inutile de piece */
  char c_lib_pie;			/* Libelle piece  */
  char c_typ_pie;			/* Type  piece  */
  void (*pf_ini_hst_mvt) (void);	/* ptr sur fonction in_mvt_XXX   */
  int (*pf_jou)  (char c_rem_pio);	/* ptr sur fonction joue pice	 */
      
  int (*pf_ech_par) (int n,int nr);     /* ptr sur fonction echec par  */
  int (*pf_ctl_par) (int n,int nr);     /* ptr sur fonction cas_tl_xxx  */
  int i_mvt_uni_dir ;    	        /* mvt unique pour direction   */
  int i_typ_mvt_tou ;                   /* type de mvt selon colone oui = 1 pour reine et tour */   
  int i_typ_mvt_fou ;                   /* type de mvt selon oblique oui= 1 pour reine et fou */

};

struct st_pie ts_pie_0[18];			/* pce camp 0			*/
struct st_pie ts_pie_1[18];			/* pice camp 1			*/

/* tableau pointant sur pice camp 0 ou 1 */
struct st_pie *tpts_pie[2];			/* poste 0 camp 0, 1 camp 1 */

/* tableau touver ID piece roi camp [0/1] */
int Id_roi[2];
/* tableau pour trouver position du roi  partir du camp */
int *pts_pie_pos_Roi[2];

/* tableau echiquier accs par l'emplacement  */
struct st_ech
{
  int i_cam;
  /* opti perf: pourrait peut etre utiliser une adresse sur piece */ 
  int i_Id;
};
struct st_ech ts_ech[100];


/* tableau historique des coups */
struct st_hst_pro
{
  int i_pro;			/* profondeur pour test du ptr pts_hst_pro */
  int i_exp_par;                /* rester au mme niveau si deplacement suit une prise */
  int i_eff_bor;                /* elimination des prise en bas  de l'arbre */
  tpf_xxx *ptpf_mvt;	        /* ptr sur fonction  mouvement */
  int i_cam;    		/* camp	*/
  struct st_pie *pts_pie_pc;    /* pointeur sur piece propre camp  */        
  int i_rap_for_ini;            /* rapport de force initiale propre camp */
  int i_cam_adv;		/* camp	 adverse */
  struct st_pie *pts_pie_ca;     /* pointeur sur piece camp adverse */
  int i_Id;                 	/* Idetifiant pice */
  /* opti : voir si stockage de l'adresse serait pas plus performant */
  int i_pos_n;                 	/* pos. chiquier*/
  int i_typ_mvt;		/* type mouvement */
  int i_dir_sui;		/* direction suivante*/
  int i_pos_ori_n;              /* pos. origine	*/
  int i_Id_2p;                  /* Id deuxime pice */
  int i_pos_n_R_p_c;	        /* Position n roi propre camp	*/
  int i_pos_n_R_c_a;	        /* Position n roi camp	adverse	*/
  int i_ech ;			/* echec par pice principale 
                                   ou secondaireou par demasquge */
  int i_ech_dir_pri;	        /* dir. pice principale ou secondaire
                                   causant echec sur camp adverse
				   (si valeur  <> 0 alors echec) */
  int i_ech_dir_dem;	        /* dir.demasaquage causant eventuel
				   echec sur camp  adverse  */
  int i_ech_dir_dem_p_c;        /* direction demasaquage causant echec	
				   sur propre camp  */
  int i_pat ;    		/* pat   */
  int i_mat ;		        /* mat 	*/
  int ti_not[2];                /* note par camp */
  int i_not;			/* note */
  int i_not_max;		/* note maximum */
  int i_rap_for;                /* rapport de force */
#if ROC_NV ==0 
  int ti_roc_cam_typ[2][2];     /* [cam][type] type: 0= petit 1= grd 
                                   Si = 1 alors piece deplace
                                   bug Suppression non signale*/
#else
  int i_ind_roc_cam_typ;        /* index sur dernier poste utilis pour mmorisation roc */

#endif
  unsigned long int uli_cpt_cou;   /* Compteur de coup */
  unsigned long int uli_cpt_mat;   /* Compteur de mat */
  unsigned long int uli_cpt_pat;   /* Compteur de pat */
  unsigned long int uli_cpt_mat_rf;/* Compteur de mat rapport de force = 0 */
  unsigned long int uli_cpt_pat_rf;/* Compteur de pat rapport de force = 0 */
  unsigned long int uli_nbr_fou;   /* Compteur de fouchettes */
  int i_nbr_pri_pie;               /* Compteur de prises faites par une piece 
				      raz  chaque chgt de piece */
  /* variable test */
# if CPT_ITE == 1
  unsigned long int cpt;
# endif
};

# if ROC_NV == 1
int ti_roc_cam_typ[2][2][PRO_HST];     /* [cam][type] type: 0= petit 1= grd */
#endif

 struct st_hst_pro ts_hst_pro[PRO_HST];

/* Tableau donnant l'ID de la tour suivant le roc et le cam */
int ti_tou_cam_typ[2][2];        /* Camp: NOIR/BLANC Typ GRA_ROC/PET_ROC */


/* Type de note pour mise en tableau */
enum typ_not
{
  nbr_mvt,
  nbr_mat,
  nbr_mat_inc,
  nbr_pat,
  nbr_pat_inc,
  nbr_mat_rf,
  nbr_pat_rf,
  nbr_fou,
  mob_pie,
  cou_pie
};
/* Nombre de type de note */
#define NBR_TYP_NOT           10   

/* Sous structure de not_mvt_ini pour evaluation mat/pat par profondeur */
struct st_hst_pro_sta	
{
  int i_cam;				/* Camp jouant a cette  pro	*/
  int i_inc;				/* Si increment profondeur = 1	*/
  float f_dat_pro [NBR_TYP_NOT];	/* Data par profondeur  non pondr */
};
struct  st_hst_pro_sta tst_pro_sta[MVT_INI][PRO_SIM_INF_MAX]; 

/* tableau note du mvt initial */
struct st_not_mvt_ini
{
  int 	i_num_mvt;	/* Numero du mvt */
  int   i_pro_sim;		/* profondeur de simulation */	 
  int	i_inc_pro_sim;		/* incrment profondeur de simulation*/
  int 	i_cam;	  	/* cam propre */
  int 	i_cam_adv;	/* cam adverse*/
  int 	i_Id;		/* pice mouvemente*/
  char 	c_lib_pie;      /*  libelle piece*/
  int 	i_typ_mvt;	/* type mouvement */
  int 	i_pos_ori_n;    /* pos d'origine chiquier */
  int 	i_pos_arr_n;    /* pos arrivee chiquier*/
  int 	i_nbr_rep_sit;  /* Nombre de rptition situation */
  int   i_not_pri;	/* Note prise */
  
  /* struct st_hst_pro_sta 	*/	/* statistique par profondeur */
  /*tst_pro_sta[PRO_SIM_INF_MAX]; */	/* marge de 1 poste car profondeur de depart */

  struct st_not
  {
    float f_not_cam_bru[2];  /* donne cumule brute(cad non pondre) */
    float f_not_cam_pon[2];  /* note pondre par camp  */
    float f_not_dif_pon;     /* difference pondre pc-ca pond */
    int   i_not_pon_rec;     /* fiffrence pondre et recadre  */
  } tst_not[NBR_TYP_NOT];

  /* Note globale */
  float f_tot_dif_pon;		/* total pondre de la note non recadre */
  float	f_tot_rec_pon;       	/* total pondr de la note recadre */ 
};

ST_NOT_MVT_INI  ts_not_mvt_ini[MVT_INI] ;

struct st_tri_mvt_ini
{
  ST_NOT_MVT_INI *pts_tri_rec;

} ts_tri_mvt_ini [MVT_INI];

/* structure pour evaluation rapport de force */
struct st_eva_sit
{
  int    ti_tot_for[2];        /* Totalisation force des piece par camp */
  int    i_not_for;            /* Force pc - ca */
  float  tf_cou[2];            /* Evaluation couverture par camp avant mouvement */
  float  f_cou;                /* cou pc - cou ca */
  float  tf_mob[2];             /* mobilit par camp */
  float  f_not_mob;              /* mob pc- mob ca */
  /* Pondration ou ordre de tri */
  int ti_coe_pon[NBR_TYP_NOT];

  /* maximum /minimu rencontre ds la selction des meilleurs mvts pour cal note recadre */

  struct st_int_not
  {
    float max;
    float min;
  } tst_int_not[NBR_TYP_NOT];


};
struct st_eva_sit  st_eva_sit ;

/* Table des messages utilisateur */
/*char tc_msg_txt[NBR_MSG][LEN_MES];*/

char *tptc_msg[MSG_NBR];

/* strucuture parametre messages */
struct st_par
{
  int i_nbr_par;                /* nombre de parametres */
  char tc_par_mes [NBR_MAX_PAR_MES][LON_MAX_PAR_MES];
};
struct st_par st_par_mes;


/* Memorisation de position des pieces detecter les cas de partie nulle */
struct st_pos
{
  int i_pos_n [2][17];		/* position cam Id  */
  int ti_roc_cam_typ[2][2];     /* [cam][type] type: 0= petit 1= grd */
  int i_cam_jou;		/* Camp qui doit jouer */
  /* info pour viter de tester l'ensemble des pieces */
  int i_nbr_pie[2];		/* Nombre de piece par camp */
  int i_som_pos[2];		/* Somme des positions */
  int i_nbr_pie_tot;		/* Nombre de pieces total */
  int i_som_pos_tot;		/* Somme des position */
}tst_pos [PRO_HST];




/*****************************************************************************/
/* Variables                                                                 */
/*****************************************************************************/
int i_cot_aff;                        	/* cote d'affichage 0= BLANC 1= NOIR */
int i_ver_eva;				/* Numro d'evaluation  */
int (*pf_eva) (void);  			/* pointeur sur fonction valuation */
int (*tpf_eva[100]) (void);  		/* tableau pointeur sur fonction valuation */
int i_aff_not_pos;                      /* =1 si affichage note possible */
int i_aff_cal_cou;			/* = 1 si affichage calcul en cour */
int i_pro_act;                          /* profondeur actuelle	*/
int i_pro_sim_niv;			/* niveau de simulation */
int i_pro_sim_inc;			/* nivde simu pour incrmen*/
int i_pro_sim_act;             		/* profondeur de simulation 		*/
int i_pro_sim_max_sup;         		/* profondeur maximale superieur 	*/
int i_pro_sim_max_inf;         		/* profondeur maximale inferieur	*/
int i_pro_sim_int_cla;         		/* profondeur d'interruption clavier*/
int i_sto_sim_cla;			/* si = 1 CTL C pendant simulation-> arret simu */
int i_sim_end;				/* si = 1 fin de simulation 		*/
int i_not_cpt_mvt_ini;			/* note: compteur mvt initial 		*/
int i_not_num_mvt_cho;			/* note n du mvt choisi 			*/
struct st_hst_pro	*pts_hst_pro; 	/* ptr sur tableau structure historique  */
struct st_hst_pro	*pts_hst_pro_p; /* ptr sur tableau structure historique  */
/* profondeur precedente 				 */
struct st_pie  		*pts_pie;      	/* ptr sur tbl structure pice principale*/

/* variable de travaille locale  */
struct st_pie  		*pts_pie_sui;    /* ptr sur tbl struct pice suivante 	 */
struct st_pie  		*pts_pie_pri;    /* ptr sur tbl struct pice prise    	 */
struct st_pie  		*pts_pie_tou_roc;/* ptr sur tbl struct pice tour roc	 */
struct st_pie  		*pts_pie_pre_pri;/* ptr tbl struct pice (prec. prise)   */
struct st_pie  		*pts_pie_sui_pri;/* ptr tableau structpice (suive prise)*/
struct st_ech		*pts_ech;	 /* ptr tableau struct echiquier		 */
int    n;				 /* position pice 				 */
int    nn;				 /* position pice 				 */
int    i_cam_adv;			 /* position pice 				 */
char   c_typ_pie;			 /* position pice 				 */

/* variable test */
# if CPT_ITE == 1
unsigned long int cpt;
# endif

/*****************************************************************************/
/*prototype de fonction                                                      */
/*****************************************************************************/
/*----------------------------------------------------------------------*/
/* initialisation et chargement tableau */
/*----------------------------------------------------------------------*/
void 	ini_pgm 	(void);
void 	ini_msg 	(void);
/* initialisation necessaire pour la simulation */
void 	ini_sim 	(void);
/*initialisation des tableaux pieces */
void 	ini_pie 	(void);
/* initialisation du tableau historique */
int 	ini_hst 	(int i_cam_hst_0);
/* initialisation du tableau echiquier */
void 	ini_ech 	(void);
/* installation des piece en dbut de partie */
void 	ins_pie_01	(void);
void 	ins_pie_02	(void);
int 	ajo_pie 	(int i_cam,char c_cod_pie,int i_pos_n);


/*--------------------------------------------------------------------*/
/* Fonction pour jouer                                                */
/*--------------------------------------------------------------------*/
/* joue interactif + reponse ordinateur  */
int jou_int_jou	 (char *ptc_dep,		
		  char *ptc_arr);

/* joue interactif + controle echec  */
int jou_int_ctl	 (char *ptc_dep,		
		  char *ptc_arr);
/* simul et joue  */
int sim_jou(void);
/* jou puis simu et jou (l'ordinateur joue apres que l'utilisateur a joue */ 
int jou_sim_jou (char *ptc_dep,		
		 char *ptc_arr);

/* joue interactif */
int jou_int 	 (char *ptc_dep,		
		  char *ptc_arr);

/* joue avec coordonnes reelles */	  
int jou		(int i_pos_dep_n,  		
		 int i_pos_arr_n,
		 char c_rem_pio	);

void jou_pro_sui	(int i_pos_dep_n,       /* initialisation profondeur suivante */
			 int i_pos_arr_n);
				 
int jou_roi 		(char c_rem_pio);	/* joue roi*/
int jou_grd_roc_0 	(void);	 		/* joue roi grd rock blanc*/
int jou_grd_roc_1 	(void);	 		/* joue roi grd rock noir */
int jou_pet_roc_0 	(void);	 		/* joue roi petit rock blanc */
int jou_pet_roc_1 	(void);	 		/* joue roi petit rock noir */

void jou_mvt_roi_roc	(int i_pos_n_tou);	/* joue roi fct commune rock	*/

int jou_ctfr		(char c_rem_pio); 	/* joue cavalier tour fou reine */
int jou_pio_0 		(char c_rem_pio); 	/* joue pion blanc */
int jou_pio_1 		(char c_rem_pio); 	/* joue pion noir  */

int jou_pio_s1c 	(void);	 		/* joue pion saut 1 case */
int jou_pio_pro 	(char c_tpie_rpio);	/* joue pion promotion */
int jou_pio_s2c 	(int i_cam); 		/* joue pion saut 2 cases */
int jou_pio_pro_pri 	(char c_tpie_rpio);	/* joue pion  promotion + prise */
int jou_pio_pri_pas 	(int i_cam); 		/* joue pion  prise en passant */
int jou_pio_pri 	(void); 		/* joue pion  prise */


int jou_mvt_pio		(void); 		/* joue pion fct commune rock */

/* Copie val grand/petit rock */
void cop_val_jou_grc_pc (void);             	/* pour propre camp */
void cop_val_jou_prc_pc (void);

void cop_val_jou_grc_ca (void);             	/* camp adverse */
void cop_val_jou_prc_ca (void);


int  ann_cou_int (void);	                /* annule coup interactif */
void ann_cou 	 (void);			/* annule coup */
void pro_pre 	 (void);			/* profondeur precedente */

int  tra_col (char c_col);      		/* pour transformer A1 en x=2,y=2 */
int  tra_lig (char c_lig);

int ctl_mvt_roi(void);                              /* controle mouvement */
int ctl_mvt_cav(void);
int ctl_mvt_tou(void);
int ctl_mvt_fou(void);
int ctl_mvt_rei(void);

void ini_pos (int i_pro);			/* init du trait */
void sav_pos_pie (int i_pro, int i_cam);	/* sauvegarde du trait piece d'un camp */
void sav_pos (int i_pro);			/* sauvegarde du trait */
int  ctl_rep_pos (int i_pro);			/* controle si trait se reproduit */		
int  ctl_rep_pos_pie (int i_pro_act, int i_pro);/* controle complet toutes pieces */ 

/* Chargement variable avec message */
int cha_mes (int i_mes_num,char *tc_lib_mes);

/* verifie  si affichage note possible */
int ver_aff_not_pos(void);

/* verifie existance mvt ini */
int ver_exi_mvt_ini(int i_mvt_ini);


/* verifie validite du libelle piece promotion du pion */
int  pro_pio_ver_lib_pie(char c_lib_pie);

/* verifie validite du type piece promotion du pion */
int  pro_pio_ver_typ_pie(char c_typ_pie);

/* traduit libelle piece en type piece */
char pro_pio_tra_lpie_tpie(char c_lib_pie);

/* traduit type mvt en type  piece*/
char pro_pio_tra_tmvt_tpie (int i_typ_mvt);

/* traduit type mvt en type  piece*/
char pro_pio_tra_tmvt_lpie (int i_typ_mvt);

/* traduit type piece en libelle piece*/
char pro_pio_tra_tpie_lpie (char  i_typ_pie);

/* Choix profondeur et incrment de simulation */
int cho_niv (int i_niv, int i_inc);


/* Sauvegarde et restauration partie sur disque */
int sav_dsk(char tc_nom_fic[40]);
int rst_dsk(char tc_nom_fic[40]);

/*-----------------------------------------------------------------*/
/* Evaluation                                                      */
/*-----------------------------------------------------------------*/
int 	eva_mei_mvt	(void);	      	 	/* retourne meilleur mouvement */
int 	eva_mei_mvt_01	(void);		       	/* retourne meilleur mouvement */
int 	eva_mei_mvt_02	(void);		       	/* retourne meilleur mouvement */
int 	eva_mei_mvt_test(void);		       	/* retourne mouvement aux hasard */

void 	not_mvt_ini_raz	(int i_pro_sim,		/* raz de la structure note mvt ini  avant simulation */
			 int i_inc_pro_sim);			
void	hst_cpt_raz	(void);			/* init nbre de coup par niveaux */
void 	not_cal 	(void);			/* calcule et mmorisation de la note du mvt initial */
void 	tra_don		(void);
void	cal_cou_rep 	(ST_NOT_MVT_INI *pts_mvt);/* calcul couverture */

float 	eva_cou_cam 	(int nn);		/* analyse de la couverture */
float 	eva_cou_pie 	(int nn,int i_cam);	/* analyse de la couverture par piece*/

void 	cal_mob 	(			/* calcul mobilite */
			 ST_NOT_MVT_INI *pts_mvt
			 );

void 	cum_not_pon 	(			/* cumul note pondr*/
			 enum  typ_not e_typ_not ,
			 ST_NOT_MVT_INI *pts_mvt
			 );
void 	cum_not 	(			/* cumul note sans pondr*/
			 enum typ_not e_typ_not,
			 ST_NOT_MVT_INI *pts_mvt
			 );

void 	cal_dif_max 	(			/* Calcul difference pc-ca et min max*/
			 enum typ_not e_typ_not,
			 ST_NOT_MVT_INI *pts_mvt 
			 );
void 	cal_max 	(			/* Calcul min max*/
			 enum typ_not e_typ_not,
			 ST_NOT_MVT_INI *pts_mvt 
			 );

void 	rec_not_mvt 	(enum typ_not e_typ_not);/* recadrage de la note en fonction des mini maxi */
int 	rec_not		(float min, float max,	 /* Recadrage note */
			 float val);
void 	pon_not		(void);			/* Pondration note */

void 	cho_coe_01 	(void);			/* Choix coeficient */
void 	cho_coe_02 	(void);			/* Choix coeficient */
int 	eva_for_cam	(int i_cam);            /* retourne  force du camp */
void 	eva_sit		(int i_cam,int i_cam_adv);
int 	ctl_ega		(void);			/* controle si galit doit tre vite */
void  	sim_aut 	(void);			/* pour faire jouer automatiquement */
void  	test_aut 	(void);			/* pour techercher d'ventuel bugs  */
int 	jou_par_aut	(int i_mvt_ini);	/* joue une partie automatiquement */ 
void  	tri_mvt_rec 	(void);			/* tri des mvt initial par ordre decroissant not_rec*/


/*-------------------------------------------------------------------------------*/
/* simulation ec_simu.h                                            		 */
/*-------------------------------------------------------------------------------*/
int  simul_ctl(void);			/* simulation pour controle mat/pat */

int  simul_eva(int i_pro_sim_niv_wrk,	/* simulation pour evaluation meilleur mvt */
	       int i_pro_sim_inc_wrk	);

int  simul(int i_pro_sim_niv_wrk,	/* retourne meilleur mouvement */
	   int i_pro_sim_inc_wrk	);
		
void sto_sim_int (void);		/* Stop simulation si touche escape */
void sto_sim 	 (void);		/* Stop simulation  */
void fct_nul     (void);                /* ne fait rien , pour viter de tester si pointeur de fonction est NULL*/

void pie_sui 	(void);                 /* piece suivante */


void dir_sui(void);                      /* Direction suivante */

void maj_not(void);
void pro_sui(void);			/* Initialisation profondeur */

/* Copie val grand/petit rock */
void cop_val_grc_pc (void);             /* pour propre camp */
void cop_val_prc_pc (void);

void cop_val_grc_ca (void);             /* camp adverse */
void cop_val_prc_ca (void);


/* initialisation hst avec pos roi camp propre et adverse*/
void hst_pos_roi (void);

/* fonctions de deplacement */
void dep_sim(void);			/* deplacement simple*/
void pri_sim(void);			/* prise simple */
void pio_rei(void);			/* pion en reine */
void pio_fou(void);			/* pion en fou */
void pio_tou(void);			/* pion en tou */
void pio_cav(void);			/* pion en cav */
void pri_pio_rei(void);			/* pion en reine + prise */
void pri_pio_fou(void);			/* pion en fou + prise 	*/
void pri_pio_tou(void);			/* pion en tou + prise */
void pri_pio_cav(void);			/* pion en cav + prise 	*/
void pri_pas(void);			/* prise ne passant */
void dep_pet_roc_0(void);               /* petit rock blanc */
void dep_pet_roc_1(void);               /* petit rock noir */
void dep_grd_roc_0(void);               /* grand rock blanc */
void dep_grd_roc_1(void);                /* grand rock noir */

/* fonction de restoration position */
void dep_res(void);			/* restore piece principale */
void pri_res(void);			/* restore pice principale + supprime */
void pie_pri_res(void);			/* restore seulement pice supprime */
void gra_roc_res(void);			/* restore piece princiaple + tour grand rock */ 
void tou_gra_roc_res(void);		/* tour grand rock */
void pet_roc_res(void);			/* restore piece principale + tour petit rock*/
void tou_pet_roc_res(void);		/* tour petit rock */
void pio_res(void);			/* restore piece principale + type de piece en pion  */
void pri_pio_res(void);			/* pion en cav,tou,fou,rei */

/* fonction modification du pion en cavalier,tour,fou,reine*/
void pio_cav_blanc(void);		/*  pion en cav */
void pio_cav_noir(void);		/*  pion en cav */
void pio_cav_2(void);			/*  pion en cav */

void pio_rei_blanc(void);		/*  pion en rei */
void pio_rei_noir(void);		/*  pion en rei */
void pio_rei_2(void);			/*  pion en rei */

void pio_tou(void);			/*  pion en tou */
void pio_fou(void);			/*  pion en fou */

void set_pie_pio(void);			/* initialise en pion */
void set_pie_pio_blanc(void);
void set_pie_pio_noir(void);

void set_pie_cav(void);			/* initialise en cavalier */
void set_pie_cav_blanc(void);
void set_pie_cav_noir(void);

void set_pie_tou(void);			/* initialise en tour */
void set_pie_tou_blanc(void);
void set_pie_tou_noir(void);

void set_pie_fou(void);				/* initialise en fou */
void set_pie_fou_blanc(void);
void set_pie_fou_noir(void);

void set_pie_rei(void);				/* initialise en reine */
void set_pie_rei_blanc(void);
void set_pie_rei_noir(void);

void set_pie_roi(void);				/* initialise en roi */
void set_pie_roi_blanc(void);
void set_pie_roi_noir(void);

void inc_for_pio(void);				/* Incremente note pour */
/* mouvement pion en*/
/* positione flag si echec ou pas echec */
void set_ech_on(void);
void set_ech_off(void);

/* initialisation hst avec pointeur sur tableau de fct mvt */
void ini_hst_mvt_roi     (void);
void ini_hst_mvt_rei     (void);
void ini_hst_mvt_tou     (void);
void ini_hst_mvt_fou     (void);
void ini_hst_mvt_cav     (void);
void ini_hst_mvt_pio_0   (void);
void ini_hst_mvt_pio_1   (void);

/* Calcul position suivant la direction */
/* pour tour, fou, reine */
void haut_gauche 	(void);
void haut_droite  	(void);
void bas_gauche 	(void);
void bas_droite  	(void);
void haut 		(void);
void bas 		(void);
void gauche 		(void);
void droite 		(void);

/* cas paticulier pour gestion possibilit du rock */
/* pour la tour si pos en coin echiquier		   */
void haut_tou_gr 	(void);
void bas_tou_gr 	(void);
void haut_tou_pr 	(void);
void bas_tou_pr 	(void);
void droite_tou_gr	(void);
void gauche_tou_pr	(void);

/* fonction commune tour fou reine */
void mvt_tfr		(void);

/* cas paticulier pour gestion possibilit du rock */
/* pour la tour si pos en coin echiquier		   */
void mvt_tou_gr		(void);
void mvt_tou_pr		(void);

/* pour cavalier */
void haut_droite_C 	(void);
void droite_haut_C 	(void);
void droite_bas_C	(void);
void bas_droite_C  	(void);
void bas_gauche_C 	(void);
void gauche_bas_C 	(void);
void gauche_haut_C	(void);
void haut_gauche_C  (void);
/* fonction commune cavalier */
void mvt_cav		(void);

/* pour roi */
void haut_gauche_R 	(void);
void haut_droite_R 	(void);
void bas_gauche_R 	(void);
void bas_droite_R  	(void);
void haut_R 		(void);
void bas_R		(void);
void gauche_R 		(void);
void droite_R 		(void);
void grd_roc_0          (void);
void pet_roc_0          (void);
void grd_roc_1          (void);
void pet_roc_1          (void);

/* fonction commune roi */
void mvt_roi		(void);
void mvt_roi_pet_roc 	(int i_pos_n_tou);
void mvt_roi_gra_roc 	(int i_pos_n_tou);

/* pour pion camp 0 */
void haut_gauche_p_0    (void);
void haut_droite_p_0    (void);
void haut_gauche_pas_p_0(void);
void haut_droite_pas_p_0(void);
void haut_gauche_cav_p_0(void);
void haut_droite_cav_p_0(void);
void haut_gauche_rei_p_0(void);
void haut_droite_rei_p_0(void);
void haut_cav_p_0       (void);
void haut_rei_p_0       (void);
void haut_1_p_0 	(void);
void haut_2_p_0		(void);

/* pour pion camp 1 */
void bas_gauche_p_1    (void);
void bas_droite_p_1    (void);
void bas_gauche_pas_p_1(void);
void bas_droite_pas_p_1(void);
void bas_gauche_cav_p_1(void);
void bas_droite_cav_p_1(void);
void bas_gauche_rei_p_1(void);
void bas_droite_rei_p_1(void);
void bas_cav_p_1       (void);
void bas_rei_p_1       (void);
void bas_1_p_1         (void);
void bas_2_p_1         (void);

/* fonction commune pion */
void mvt_pio_pro 	( void (*pf_set_pie) (void),int i_typ_mvt);
void mvt_pio_pro_pri 	( void (*pf_set_pie) (void),int i_typ_mvt);
void mvt_pio_pri_pas 	(int i_inc_ori,int i_inc_arr);
void mvt_pio_pri	(void);
void mvt_pio_1C		(void);
void mvt_pio_2C		(int i_dec_pos);
void mvt_pio		(int i_pri_pas);

int  get_key (void);
void sto_sim_set (int i_sig);
void sto_sim_int (void);
#if CTL_DEP_ECH_PIE == 1
int ctl_dep_nbr(void);   /* controle dephasage nombre de pieces */
#endif

/*-----------------------------------------------------------------------*/
/* ec_ctl.c								 */
/*-----------------------------------------------------------------------*/
/* controle si la piece  mouvementee cause un echec*/
int ech_par_rei     (int n, int nr);
int ech_par_fou     (int n, int nr);
int ech_par_tou     (int n, int nr);
int ech_par_cav     (int n, int nr);
int ech_par_roi     (int n, int nr);
int ech_par_pio_0   (int n, int nr);
int ech_par_pio_1   (int n, int nr);

/* controle si une case est controlee par une piece*/
int ctl_par_rei     (int n, int nr);
int ctl_par_fou     (int n, int nr);
int ctl_par_tou     (int n, int nr);
int ctl_par_cav     (int n, int nr);
int ctl_par_roi     (int n, int nr);
int ctl_par_pio_0   (int n, int nr);
int ctl_par_pio_1   (int n, int nr);

int ctl_par_xxx(int nn);

/* echec: calcul direction demasquage */
void ech_cal_dir_dem  (int i_cam_roi,int *pi_ech_dir_dem);
void ech_cal_dir_dem_ca  (void);
void ech_cal_dir_dem_pc  (void);


/* adaptation pour traitement prise en passant */
int  ech_cal_dir_dem_2  (int i_cam_roi,int i_pos_n);

/* fct pour controle roi propre camp  echec     */
/* oblique: fou et reine */
int cas_ctl_nul     (int n,int i_cam_adv);
int cas_ctl_h_d     (int n,int i_cam_adv);
int cas_ctl_h_g     (int n,int i_cam_adv);
int cas_ctl_b_d     (int n,int i_cam_adv);
int cas_ctl_b_g     (int n,int i_cam_adv);

/* droite  tour et reine */
int cas_ctl_h     (int n,int i_cam_adv);
int cas_ctl_b     (int n,int i_cam_adv);
int cas_ctl_d     (int n,int i_cam_adv);
int cas_ctl_g     (int n,int i_cam_adv);

/* cavalier */
int cas_ctl_cav_h_d     (int n,int i_cam_adv);
int cas_ctl_cav_d_h     (int n,int i_cam_adv);
int cas_ctl_cav_d_b     (int n,int i_cam_adv);
int cas_ctl_cav_b_d     (int n,int i_cam_adv);
int cas_ctl_cav_b_g     (int n,int i_cam_adv);
int cas_ctl_cav_g_b     (int n,int i_cam_adv);
int cas_ctl_cav_g_h     (int n,int i_cam_adv);
int cas_ctl_cav_h_g     (int n,int i_cam_adv);

/* pion */
int cas_ctl_pio_g     	(int n,int i_cam_adv);
int cas_ctl_pio_d     	(int n,int i_cam_adv);




/* constante tableau mouvemnt */

/* intialisation ptr fonction  suivant position roi */
/* roi haut gauche */
/* static const  void (*tpf_roi_h_g	)(void)[] =*/
static const tpf_xxx tpf_roi_h_g [] =
{
  droite_R ,
  bas_droite_R,
  bas_R,
  pie_sui
};

/* roi haut */
static const tpf_xxx tpf_roi_h	[] =
{
  droite_R,
  bas_droite_R,
  bas_R,
  bas_gauche_R,
  gauche_R,
  pie_sui
};
/* roi pos origine noir */
/* laisser grd et petit roc en fin de tab pour eviter restore de piece */
static const tpf_xxx tpf_roi_pos_ori_1 [] =
{
  droite_R,
  bas_droite_R,
  bas_R,
  bas_gauche_R,
  gauche_R,
  grd_roc_1, /* laisser grd et petit roc en fin de tableau */
  pet_roc_1,
  pie_sui
};
/* roi haut droite */
static const tpf_xxx tpf_roi_h_d[] =
{
  bas_R,
  bas_gauche_R,
  gauche_R,
  pie_sui
};
/* roi droite */
static const tpf_xxx tpf_roi_d[] =
{
  haut_R,
  bas_R,
  bas_gauche_R,
  gauche_R,
  haut_gauche_R,
  pie_sui
};

/* roi bas droite */
static const tpf_xxx tpf_roi_b_d[] =
{
  haut_R,
  gauche_R,
  haut_gauche_R,
  pie_sui
};

/* roi bas */
static const tpf_xxx tpf_roi_b[] =
{
  haut_R,
  haut_droite_R,
  droite_R,
  gauche_R,
  haut_gauche_R,
  pie_sui
};

/* roi pos origine blanc */
/* laisser grd et petit roc en fin de tab pour eviter restore de piece */
static const tpf_xxx tpf_roi_pos_ori_0[] =
{
  haut_R,
  haut_droite_R,
  droite_R,
  gauche_R,
  haut_gauche_R,
  grd_roc_0,	  /* laisser grd et petit roc en fin de tableau */
  pet_roc_0,
  pie_sui
};
/* roi bas gauche */
static const tpf_xxx tpf_roi_b_g[] =
{
  haut_R,
  haut_droite_R,
  droite_R,
  pie_sui
};

/* roi gauche */
static const tpf_xxx tpf_roi_g[] =
{
  haut_R,
  haut_droite_R,
  droite_R,
  bas_droite_R,
  bas_R,
  pie_sui
};

/* roi normal */
static const tpf_xxx tpf_roi_nor[] =
{
  haut_R,
  haut_droite_R,
  droite_R,
  bas_droite_R,
  bas_R,
  bas_gauche_R,
  gauche_R,
  haut_gauche_R,
  pie_sui
};
/* initialisation ptr tbl ptr mvt selon position */

static const tpf_xxx *ptpf_mvt_roi[] =
{
#include "gen/ec_mvt_r.i"
};


/* reine haut gauche */
static const tpf_xxx tpf_rei_h_g[]=
{
  droite,
  bas_droite,
  bas,
  pie_sui
};
/* reine haut */


static const tpf_xxx tpf_rei_h[] =
{
  droite,
  bas_droite,
  bas,
  bas_gauche,
  gauche,
  pie_sui
};
/* reine haut droite */
static const tpf_xxx tpf_rei_h_d[] =
{
  bas,
  bas_gauche,
  gauche,
  pie_sui
};
/* reine droite */
static const tpf_xxx tpf_rei_d[] =
{
  haut,
  bas,
  bas_gauche,
  gauche,
  haut_gauche,
  pie_sui
};
/* reine bas droite */
static const tpf_xxx tpf_rei_b_d[] =
{
  haut,
  gauche,
  haut_gauche,
  pie_sui
};
/* reine bas */
static const tpf_xxx tpf_rei_b[] =
{
  haut,
  haut_droite,
  droite,
  gauche,
  haut_gauche,
  pie_sui
};
/* reine bas gauche */
static const tpf_xxx tpf_rei_b_g[] =
{
  haut,
  haut_droite,
  droite,
  pie_sui
};
/* reine gauche */
static const tpf_xxx tpf_rei_g[] =
{
  haut,
  haut_droite,
  droite,
  bas_droite,
  bas,
  pie_sui
};
/* reine normal */
static const tpf_xxx tpf_rei_nor[] =
{
  haut,
  haut_droite,
  droite,
  bas_droite,
  bas,
  bas_gauche,
  gauche,
  haut_gauche,
  pie_sui
};
/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_rei[] =
{
#include "gen/ec_mvt_d.i"
};

/* fou haut gauche */
static const tpf_xxx tpf_fou_h_g[] =
{

  bas_droite,
  pie_sui
};
/* fou haut */
static const tpf_xxx tpf_fou_h[] =
{
  bas_droite,
  bas_gauche,
  pie_sui
};
/* fou haut droite */
static const tpf_xxx tpf_fou_h_d[] =
{
  bas_gauche,
  pie_sui
};
/* fou droite */
static const tpf_xxx tpf_fou_d[] =
{
  bas_gauche,
  haut_gauche,
  pie_sui
};
/* fou bas droite */
static const tpf_xxx tpf_fou_b_d[] =
{
  haut_gauche,
  pie_sui
};
/* fou bas */
static const tpf_xxx tpf_fou_b[] =
{
  haut_droite,
  haut_gauche,
  pie_sui
};
/* fou bas gauche */
static const tpf_xxx tpf_fou_b_g[] =
{
  haut_droite,
  pie_sui
};
/* fou gauche */
static const tpf_xxx tpf_fou_g[] =
{
  haut_droite,
  bas_droite,
  pie_sui
};
/* fou normal */
static const tpf_xxx tpf_fou_nor[] =
{
  haut_droite,
  bas_droite,
  bas_gauche,
  haut_gauche,
  pie_sui
};
/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_fou[] =
{
#include "gen/ec_mvt_f.i"
};

/* tour haut gauche */
static const tpf_xxx tpf_tou_h_g[] =
{
  droite_tou_gr,
  bas_tou_gr,
  pie_sui
};

/* tour haut */
static const tpf_xxx  tpf_tou_h[]  =
{
  droite,
  bas,
  gauche,
  pie_sui
};
/* tour haut droite */
static const tpf_xxx tpf_tou_h_d[]  = 
{
  bas_tou_pr,
  gauche_tou_pr,
  pie_sui
};

/* tour droite */
static const tpf_xxx  tpf_tou_d[]  = 
{
  haut,
  bas,
  gauche,
  pie_sui
};
/* tour bas droite */
static const tpf_xxx  tpf_tou_b_d[]  =
{
  haut_tou_pr,
  gauche_tou_pr,
  pie_sui
};

/* tour bas */
static const tpf_xxx tpf_tou_b[]  =
{
  haut,
  droite,
  gauche,
  pie_sui
};

/* tour bas gauche */
static const tpf_xxx tpf_tou_b_g[]  = 
{
  haut_tou_gr,
  droite_tou_gr,
  pie_sui
};

/* tour gauche */
static const tpf_xxx tpf_tou_g[]  = 
{
  haut,
  droite,
  bas,
  pie_sui
};

/* tour normal */
static const tpf_xxx tpf_tou_nor[]  =
{
  haut,
  droite,
  bas,
  gauche,
  pie_sui
};
 
/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_tou[] =
{
#include "gen/ec_mvt_t.i"
};


/* intialisation ptr fonction suivant position cavalier */
/* cavalier coin haut gauche */
static const tpf_xxx  tpf_cav_c_h_g[] =
{
  droite_bas_C,
  bas_droite_C,
  pie_sui
};
/* cavalier  haut gauche */
static const tpf_xxx  tpf_cav_h_g[] =
{ 
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  pie_sui
};
/* cavalier haut */
static const tpf_xxx tpf_cav_h[] =
{ 
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  pie_sui
};
/* cavalier haut droit */
static const tpf_xxx tpf_cav_h_d[] =
{
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  pie_sui
};
/* cavalier coin haut droit */
static const tpf_xxx tpf_cav_c_h_d[] =
{
  bas_gauche_C,
  gauche_bas_C,
  pie_sui
};
/* cavalier droite haut */
static const tpf_xxx   tpf_cav_d_h[] =
{
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  pie_sui
};
/* cavalier droite */
static const tpf_xxx tpf_cav_d[] =
{
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier droite bas */
static const tpf_xxx tpf_cav_d_b[] =
{
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier coin bas droite */
static const tpf_xxx tpf_cav_c_b_d[] =
{
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier bas droite */
static const tpf_xxx tpf_cav_b_d[] =
{
  haut_droite_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier bas */
static const tpf_xxx tpf_cav_b[] =
{
  haut_droite_C,
  droite_haut_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier bas gauche */
static const tpf_xxx tpf_cav_b_g[] =
{
  haut_droite_C,
  droite_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier coin bas gauche */
static const tpf_xxx tpf_cav_c_b_g[] =
{
  haut_droite_C,
  droite_haut_C,
  pie_sui
};
/* cavalier gauche bas */
static const tpf_xxx tpf_cav_g_b[] =
{ haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  pie_sui
};
/* cavalier gauche */
static const tpf_xxx tpf_cav_g[] =
{ haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  pie_sui
};
/* cavalier gauche haut */
static const tpf_xxx tpf_cav_g_h[] =
{ droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  pie_sui
};
/* cavalier interne coin haut gauche */
static const tpf_xxx tpf_cav_i_c_h_g[] =
{ 
  droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  pie_sui
};
/* cavalier interne haut */
static const tpf_xxx tpf_cav_i_h[] =
{
  droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  pie_sui
};
/* cavalier interne coin haut droit */
static const tpf_xxx tpf_cav_i_c_h_d[] =
{
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  pie_sui
};
/* cavalier interne droit */
static const tpf_xxx tpf_cav_i_d[] =
{ haut_droite_C,
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier interne coin bas droite */
static const tpf_xxx tpf_cav_i_c_b_d[] =
{
  haut_droite_C,
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier interne bas */
static const tpf_xxx tpf_cav_i_b[] =
{
  haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier interne coin gauche bas */
static const tpf_xxx tpf_cav_i_c_b_g[] =
{
  haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier interne gauche */
static const tpf_xxx tpf_cav_i_g[] =
{
  haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  haut_gauche_C,
  pie_sui
};
/* cavalier normal */
static const tpf_xxx tpf_cav_nor[] =
{
  haut_droite_C,
  droite_haut_C,
  droite_bas_C,
  bas_droite_C,
  bas_gauche_C,
  gauche_bas_C,
  gauche_haut_C,
  haut_gauche_C,
  pie_sui
};
/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_cav[] =
{
#include "gen/ec_mvt_c.i"
};


/* intialisation ptr fonction  suivant position pion */
/* laisser haut_2_p_0 sur premier poste  pour viter restore de piece avant mvt */            
/* pion en ligne 2 gauche */
static const tpf_xxx 	 tpf_pio_0_L2_g[]  =
{
  haut_2_p_0, /* a laisser toujours sur post 0 */
  haut_1_p_0,
  haut_droite_p_0,
  pie_sui
};

/* pion en ligne 2 milieu */
static const tpf_xxx tpf_pio_0_L2[]  =
{
  haut_2_p_0,
  haut_1_p_0,
  haut_gauche_p_0,
  haut_droite_p_0,
  pie_sui
};
/* pion en ligne 2 droite */
static const tpf_xxx tpf_pio_0_L2_d[]  =
{
  haut_2_p_0,
  haut_1_p_0,
  haut_gauche_p_0,
  pie_sui
};
/* pion en ligne 3,4,6 gauche */
static const tpf_xxx tpf_pio_0_L346_g[]  =
{
  haut_1_p_0,
  haut_droite_p_0,
  pie_sui
};
/* pion en ligne 3,4,6 milieu */
static const tpf_xxx tpf_pio_0_L346[]  =
{
  haut_1_p_0,
  haut_gauche_p_0,
  haut_droite_p_0,
  pie_sui
};
/* pion en ligne 3,4,6 droite */
static const tpf_xxx tpf_pio_0_L346_d[]  = 
{
  haut_1_p_0,
  haut_gauche_p_0,
  pie_sui
};
/* pion en ligne 5 gauche */
static const tpf_xxx  tpf_pio_0_L5_g[]  =
{ 
  haut_1_p_0,
  haut_droite_p_0,
  haut_droite_pas_p_0,
  pie_sui
};
/* pion en ligne 5 milieu */
static const tpf_xxx  tpf_pio_0_L5[]  =
{
  haut_1_p_0,
  haut_gauche_p_0,
  haut_droite_p_0,
  haut_gauche_pas_p_0,
  haut_droite_pas_p_0,
  pie_sui
};
/* pion en ligne 5 droite */
static const tpf_xxx  tpf_pio_0_L5_d[]  =
{
  haut_1_p_0,
  haut_gauche_p_0,
  haut_gauche_pas_p_0,
  pie_sui
};
/* conserver l'ordre des postes dans le tableau L7 */
/* car pice remis a position origine aprs piece suivante */
/* pion en ligne 7 gauche */
static const tpf_xxx tpf_pio_0_L7_g[]  =
{
  haut_cav_p_0,
  haut_rei_p_0,
  haut_droite_cav_p_0,
  haut_droite_rei_p_0,
  pie_sui
};
/* pion en ligne 7 milieu */
static const tpf_xxx  tpf_pio_0_L7[]  =
{
  haut_cav_p_0,
  haut_rei_p_0,
  haut_droite_cav_p_0,
  haut_gauche_cav_p_0,
  haut_droite_rei_p_0,
  haut_gauche_rei_p_0,
  pie_sui
};
/* pion en ligne 7 droite */
static const tpf_xxx tpf_pio_0_L7_d[]  = 
{
  haut_cav_p_0,
  haut_rei_p_0,
  haut_gauche_cav_p_0,
  haut_gauche_rei_p_0,
  pie_sui
};

/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_pio_0[] =
{
#include "gen/ec_mvtp0.i"
};

/* intialisation ptr fonction  suivant position pion */
/* laisser haut_2_p_0 sur premier poste  pour viter restore de piece avant mvt */            
/* pion en ligne 7 gauche */
static const tpf_xxx  tpf_pio_1_L7_g[]  =
{
  bas_2_p_1,   /* laisser sur poste  0 */
  bas_1_p_1,
  bas_droite_p_1,
  pie_sui
};

/* pion en ligne 7 milieu */
static const tpf_xxx tpf_pio_1_L7[]  =
{
  bas_2_p_1,
  bas_1_p_1,
  bas_gauche_p_1,
  bas_droite_p_1,
  pie_sui
};

/* pion en ligne 7 droite */
static const tpf_xxx tpf_pio_1_L7_d[]  =
{
  bas_2_p_1,
  bas_1_p_1,
  bas_gauche_p_1,
  pie_sui
};

/* pion en ligne 6,5,3 gauche */
static const tpf_xxx tpf_pio_1_L653_g[]  =
{
  bas_1_p_1,
  bas_droite_p_1,
  pie_sui
};

/* pion en ligne 6,5,3 milieu */
static const tpf_xxx tpf_pio_1_L653[]  =
{
  bas_1_p_1,
  bas_gauche_p_1,
  bas_droite_p_1,
  pie_sui
};

/* pion en ligne 6,5,3 droite */
static const tpf_xxx tpf_pio_1_L653_d[]  =
{
  bas_1_p_1,
  bas_gauche_p_1,
  pie_sui
};

/* pion en ligne 4 gauche */
static const tpf_xxx tpf_pio_1_L4_g[]  =
{
  bas_1_p_1,
  bas_droite_p_1,
  bas_droite_pas_p_1,
  pie_sui
};

/* pion en ligne 4 milieu */
static const tpf_xxx tpf_pio_1_L4[]  =
{
  bas_1_p_1,
  bas_gauche_p_1,
  bas_droite_p_1,
  bas_gauche_pas_p_1,
  bas_droite_pas_p_1,
  pie_sui
};

/* pion en ligne 4 droite */
static const tpf_xxx tpf_pio_1_L4_d[]  =
{
  bas_1_p_1,
  bas_gauche_p_1,
  bas_gauche_pas_p_1,
  pie_sui
};

/* conserver l'ordre des postes dans le tableau L7 */
/* car pice remis a position origine aprs piece suivante */
/* pion en ligne 2 gauche */
static const tpf_xxx tpf_pio_1_L2_g[]  =
{ 
  bas_cav_p_1,
  bas_droite_cav_p_1,
  bas_rei_p_1,
  bas_droite_rei_p_1,
  pie_sui
};

/* pion en ligne 2 milieu */
static const tpf_xxx tpf_pio_1_L2[]  =
{
  bas_cav_p_1,
  bas_droite_cav_p_1,
  bas_gauche_cav_p_1,
  bas_rei_p_1,
  bas_droite_rei_p_1,
  bas_gauche_rei_p_1,
  pie_sui
};

/* pion en ligne 2 droite */
static const tpf_xxx tpf_pio_1_L2_d[]  =
{
  bas_cav_p_1,
  bas_gauche_cav_p_1,
  bas_rei_p_1,
  bas_gauche_rei_p_1,
  pie_sui
};
/* initialisation ptr tbl ptr mvt selon position */
static const tpf_xxx *ptpf_mvt_pio_1[] =
{
#include "gen/ec_mvtp1.i"
};


/*  pour detection echec ec_ctl.c */
static const int ti_ech_dir_rei[100][100]=
{
#include "gen/ec_echre.i"
}; 
static const int ti_ech_dir_tou[100][100]=
{ 
#include "gen/ec_echto.i"
}; 
static const int ti_ech_dir_fou[100][100]=
{
#include "gen/ec_echfo.i"
};  
static  const int ti_ech_dir_cav[100][100]=
{
#include "gen/ec_echca.i"
}; 
static  const int ti_ech_dir_roi[100][100]=
{
#include "gen/ec_echro.i"
}; 
static  const int ti_ech_dir_pio_noi[100][100]=
{
#include "gen/ec_echpn.i"
}; 
static  const int ti_ech_dir_pio_bla[100][100]=
{
#include "gen/ec_echpb.i"
}; 
static  const int ti_ech_dir_inc[100][100]=
{
#include "gen/ec_echin.i"
}; 



/* tableau de ptr fonction restore echiquier */
/*pour chaque type de mouvement  */
/* restore piece complet  */
/* const provoque warning initialisation from incompatible pointer type */

static  void (*tpf_res_pie[NBR_TYP_MVT]) (void)=
{
  /* PAS_DE_MVT		*/ 	NULL,
  /* TFR_DEP		*/	dep_res,
  /* TFR_PRI		*/ 	pri_res,
  /* ROI_DEP		*/	dep_res,
  /* ROI_PRI		*/	pri_res,
  /* ROI_GRA_ROC	*/	gra_roc_res,
  /* ROI_PET_ROC	*/	pet_roc_res,
  /* CAV_DEP		*/ 	dep_res,
  /* CAV_PRI		*/	pri_res,
  /* PIO_1C		*/	dep_res,
  /* PIO_2C		*/ 	dep_res,
  /* PIO_PRI		*/	pri_res,
  /* PIO_PRI_PAS	*/ 	pri_res,
  /* PIO_PRO_REI	*/	pio_res,
  /* PIO_PRO_FOU	*/ 	pio_res,
  /* PIO_PRO_TOU	*/	pio_res,
  /* PIO_PRO_CAV	*/ 	pio_res,
  /* PIO_PRO_REI_PRI	*/	pri_pio_res,
  /* PIO_PRO_FOU_PRI	*/	pri_pio_res,
  /* PIO_PRO_TOU_PRI	*/	pri_pio_res,
  /* PIO_PRO_CAV_PRI	*/	pri_pio_res
};


/* 1 si prise pice */
static const int ti_typ_mvt_pri[NBR_TYP_MVT]=
{
  /* PAS_DE_MVT 	*/ 0,
  /* TFR_DEP		*/ 0,
  /* TFR_PRI		*/ 1,
  /* ROI_DEP		*/ 0,
  /* ROI_PRI		*/ 1,
  /* ROI_GRA_ROC	*/ 0,
  /* ROI_PET_ROC	*/ 0,
  /* CAV_DEP		*/ 0,
  /* CAV_PRI		*/ 1,
  /* PIO_1C		*/ 0,
  /* PIO_2C		*/ 0,
  /* PIO_PRI		*/ 1,
  /* PIO_PRI_PAS	*/ 1,
  /* PIO_PRO_REI	*/ 0,
  /* PIO_PRO_FOU	*/ 0,
  /* PIO_PRO_TOU	*/ 0,
  /* PIO_PRO_CAV	*/ 0,
  /* PIO_PRO_REI_PRI	*/ 1,
  /* PIO_PRO_FOU_PRI	*/ 1,
  /* PIO_PRO_TOU_PRI	*/ 1,
  /* PIO_PRO_CAV_PRI	*/ 1
};		

/* chargement du tableau (statique) controle  nouveau echec demasquage*/
/* const provoque warning initialisation from incompatible pointer type */
static  int (*tpf_cas_ctl[19])     (int n,int i_cam_adv)=
{
  /* 0 */ cas_ctl_nul,                /* appell pour controle echec demasquage  prise en passant 
							de la fct jou_mvt_pio: Ne peu tester si la direction est 
							= 0 (elle est recalcule dans test if) et donc l'index 0
							du tableau de pointeur peut tre appel  */	
  /*  1 */ cas_ctl_h_d,
  /*  2 */ cas_ctl_h_g,
  /*  3 */ cas_ctl_b_d,
  /*  4 */ cas_ctl_b_g,

  /*  5 */ cas_ctl_h,
  /*  6 */ cas_ctl_b,
  /*  7 */ cas_ctl_d,
  /*  8 */ cas_ctl_g,

  /*  9 */ cas_ctl_cav_h_d,
  /* 10 */ cas_ctl_cav_d_h,
  /* 11 */ cas_ctl_cav_d_b,
  /* 12 */ cas_ctl_cav_b_d,
  /* 13 */ cas_ctl_cav_b_g,
  /* 14 */ cas_ctl_cav_g_b,
  /* 15 */ cas_ctl_cav_g_h,
  /* 16 */ cas_ctl_cav_h_g,

  /* 17 */ cas_ctl_pio_d,
  /* 18 */ cas_ctl_pio_g

};

/* tableau correspondance posx <-> libelle colonne */
static const char tc_lib_cas[100] [3]=
{
  /* 0 */ "  ",
  /* 1 */"  ",
  /* 2 */ "  ",
  /* 3 */ "  ",
  /* 4 */ "  ",
  /* 5 */ "  ",
  /* 6 */ "  ",
  /* 7 */ "  ",
  /* 8 */ "  ",
  /* 9 */ "  ",

  /* 10 */ "  ",
  /* 11 */ "A1",
  /* 12 */ "B1",
  /* 13 */ "C1",
  /* 14 */ "D1",
  /* 15 */ "E1",
  /* 16 */ "F1",
  /* 17 */ "G1",
  /* 18 */ "H1",
  /* 19 */ "  ",

  /* 20 */ "  ",
  /* 21 */ "A2",
  /* 22 */ "B2",
  /* 23 */ "C2",
  /* 24 */ "D2",
  /* 25 */ "E2",
  /* 26 */ "F2",
  /* 27 */ "G2",
  /* 28 */ "H2",
  /* 29 */ "  ",

  /* 30 */ "  ",
  /* 31 */ "A3",
  /* 32 */ "B3",
  /* 33 */ "C3",
  /* 34 */ "D3",
  /* 35 */ "E3",
  /* 36 */ "F3",
  /* 37 */ "G3",
  /* 38 */ "H3",
  /* 39 */ "  ",

  /* 40 */ "  ",
  /* 41 */ "A4",
  /* 42 */ "B4",
  /* 43 */ "C4",
  /* 44 */ "D4",
  /* 45 */ "E4",
  /* 46 */ "F4",
  /* 47 */ "G4",
  /* 48 */ "H4",
  /* 49 */ "  ",

  /* 50 */ "  ",
  /* 51 */ "A5",
  /* 52 */ "B5",
  /* 53 */ "C5",
  /* 54 */ "D5",
  /* 55 */ "E5",
  /* 56 */ "F5",
  /* 57 */ "G5",
  /* 58 */ "H5",
  /* 59 */ "  ",
	
  /* 60 */ "  ",
  /* 61 */ "A6",
  /* 62 */ "B6",
  /* 63 */ "C6",
  /* 64 */ "D6",
  /* 65 */ "E6",
  /* 66 */ "F6",
  /* 67 */ "G6",
  /* 68 */ "H6",
  /* 69 */ "  ",

  /* 70 */ "  ",
  /* 71 */ "A7",
  /* 72 */ "B7",
  /* 73 */ "C7",
  /* 74 */ "D7",
  /* 75 */ "E7",
  /* 76 */ "F7",
  /* 77 */ "G7",
  /* 78 */ "H7",
  /* 79 */ "  ",

  /* 80 */ "  ",
  /* 81 */ "A8",
  /* 82 */ "B8",
  /* 83 */ "C8",
  /* 84 */ "D8",
  /* 85 */ "E8",
  /* 86 */ "F8",
  /* 87 */ "G8",
  /* 88 */ "H8",
  /* 89 */ "  ",

  /* 90 */ "  ",
  /* 91 */ "  ",
  /* 92 */ "  ",
  /* 93 */ "  ",
  /* 94 */ "  ",
  /* 95 */ "  ",
  /* 96 */ "  ",
  /* 97 */ "  ",
  /* 98 */ "  ",
  /* 99 */ "  "	
};

